// Copyright (C) 2020 Guido Berhoerster <guido+sievemgr@berhoerster.name>
//
// Permission is hereby granted, free of charge, to any person obtaining
// a copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to
// permit persons to whom the Software is furnished to do so, subject to
// the following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
// IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
// CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
// TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
// SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

package main

import (
	"fmt"
	"io"
	"os"

	"go.guido-berhoerster.org/managesieve"
)

func init() {
	cmdPut.Flag.StringVar(&acctName, "a", "", "Select the account")
	cmdCheck.Flag.StringVar(&acctName, "a", "", "Select the account")
}

var cmdPut = &command{
	UsageLine: "put [options] name [file]",
	Run:       runPut,
}

var cmdCheck = &command{
	UsageLine: "check [options] [file]",
	Run:       runPut,
}

func runPut(cmd *command, args []string) error {
	var err error
	acct, err := getAccount(&conf, acctName)
	if err != nil {
		return err
	}

	var scriptName string
	var r io.Reader = os.Stdin
	if cmd.Name() == "put" {
		switch len(args) {
		case 2: // name and filename
			scriptFile, err := os.Open(args[1])
			if err != nil {
				return fmt.Errorf("failed to open script file: %s\n",
					err)
			}
			defer scriptFile.Close()
			r = scriptFile
			fallthrough
		case 1: // only name
			scriptName = args[0]
		default:
			return usageError("invalid number of arguments")
		}
	} else if cmd.Name() == "check" {
		switch len(args) {
		case 1: // filename
			scriptFile, err := os.Open(args[0])
			if err != nil {
				return fmt.Errorf("failed to open script file: %s\n",
					err)
			}
			defer scriptFile.Close()
			r = scriptFile
		case 0:
		default:
			return usageError("invalid number of arguments")
		}
	}

	script, err := readLimitedString(r, managesieve.ReadLimit)
	if err != nil {
		return fmt.Errorf("failed to read script: %s\n", err)
	}

	if err := lookupHostPort(acct); err != nil {
		return err
	}

	if err := readPassword(acct); err != nil {
		return err
	}

	c, err := dialPlainAuth(acct)
	if err != nil {
		return err
	}
	defer c.Logout()

	var warnings string
	if cmd.Name() == "put" {
		warnings, err = c.PutScript(scriptName, script)
	} else if cmd.Name() == "check" {
		warnings, err = c.CheckScript(script)
	}
	if err != nil {
		return err
	} else if warnings != "" {
		fmt.Fprintln(os.Stderr, warnings)
	}

	return nil
}
